#include "cquad.h"

double cquad_dqawf(double f(), double a, double omega, int integr, double epsabs, double *abserr, int *neval, int *ier, int *lst, int *iwork, double *work)
{
/******************************************************************************
***  Date written:         03/31/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Last revision date:   11/02/2012  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Author:    Gianluca Russo    (gianlurusso@fastwebnet.it)               ***
***             Salvatore Cuomo   (salvatore.cuomo@unina.it)                ***
***                                                                         ***
***             University of Naples "Federico II"                          ***
***             Dipartimento di Matematica e Applicazioni "R. Caccioppoli"  ***
***									    ***
*** 									    ***
***									    ***
*** This code is provided "as is", without any warranty of any kind,        ***
*** either expressed or implied, including but not limited to, any implied  ***
*** warranty of merchantibility or fitness for any purpose. In no event     ***
*** will any party who distributed the code be liable for damages or for    ***
*** any claim(s) by any other party, including but not limited to, any      ***
*** lost profits, lost monies, lost data or data rendered inaccurate,	    ***
*** losses sustained by third parties, or any other special, incidental or  ***	
*** consequential damages arising out of the use or inability to use the    ***
*** program, even if the possibility of such damages has been advised	    ***	
*** against. The entire risk as to the quality, the performance, and the    ***	
*** fitness of the program for any particular purpose lies with the party   ***
*** using the code.							    ***
***									    ***
*** This code, and any derivative of this code, may not be used in a        ***
*** commercial package without the prior explicit written permission of     ***
*** the authors. Verbatim copies of this code may be made and distributed   ***
*** in any medium, provided that this copyright notice is not removed or    ***
*** altered in any way. No fees may be charged for distribution of the      ***
*** codes, other than a fee to cover the cost of the media and a            ***
*** reasonable handling fee.						    ***
***-------------------------------------------------------------------------***
***  Purpose:                                                               ***
***                                                                         ***
***    The function provides a C interface to the fortran routine dqawf     ***
***    of quadpack package.                                                 ***
***                                                                         ***
***    The function calculates an approximation result to a given fourier   ***
***    integral i = integral of f(x) * w(x) over (a, infinity), where:      ***
***                                                                         ***
***                    w(x) = cos(omega * x)   or                           ***
***                    w(x) = sin(omega * x)                                ***
***                                                                         ***
***    hopefully satisfying following claim for accuracy:                   ***
***                                                                         ***
***                     abs(i - result) <= epsabs                           ***
***-------------------------------------------------------------------------***
***  Description:                                                           ***
***                                                                         ***
***    Calculates the Fourier cosine or Fourier sine transform of f(X), for ***
***    user-supplied interval (a, infinity), omega, and f. The procedure of ***
***    cquad_dqawo is used on successive finite intervals, and convergence  ***
***    acceleration by means of the Epsilon algorithm is applied to the     ***
***    series of the integral contributions.                                ***
***                                                                         ***
***    Double precision version.                                            ***
***-------------------------------------------------------------------------***
***  Input:                                                                 ***
***                                                                         ***
***    f        function subprogram defining the integrand function f(x)    ***
***                                                                         ***
***    a        lower limit of integration                                  ***
***                                                                         ***
***    omega    parameter in the integrand weight function                  ***
***                                                                         ***
***    integr   indicates which of the weight functions is used:            ***
***                                                                         ***
***                 integr = 1 -->     w(x) = cos(omega * x)                ***
***                 integr = 2 -->     w(x) = sin(omega * x)                ***
***                                                                         ***
***    epsabs   absolute accoracy requested                                 ***
***-------------------------------------------------------------------------***
***  Output:                                                                ***
***                                                                         ***
***    res      approximation to the integral                               ***
***                                                                         ***
***    abserr   estimate of the modulus of the absolute error               ***
***                                                                         ***
***    neval    number of integrand evaluations                             ***
***                                                                         ***
***    ier      normal termination flag:                                    ***
***                                                                         ***
***               ier = 0 --> normal and reliable termination of the        ***
***                           function. It is assumed that the requested    ***
***                           accuracy has been achieved.                   ***
***                                                                         ***
***              if omega != 0:                                             ***
***                                                                         ***
***               ier = 1 --> maximum number of cycles allowed has been     ***
***                           achieved.                                     ***
***                                                                         ***
***               ier = 4 --> the extrapolation table constructed for       ***
***                           convergence accelaration of the series formed ***
***                           by the integral contributions over the        *** 
***                           cycles does not converge to within the        ***
***                           requested accuracy.                           ***
***                                                                         ***
***               ier = 6 --> the input is invalid because epsabs <= 0, or  ***
***                           (integr != 1 and integr != 2), or limlst < 1, ***
***                           or leniw < (limlst + 2), or maxp1 < 1,  or    ***
***                           lenw < (leniw * 2 + maxp1 * 25).              ***
***                           In all these events res, abserr, neval and    ***
***                           lst are set to zero.                          ***
***                                                                         ***
***               ier = 7 --> bad integrand behaviour occurs within one or  ***
***                           more of the cycles. location and type of the  ***
***                           difficulty involved can be determined from    ***
***                           the first lst elements of vector iwork. Here  ***
***                           lst is the number of cycles actually needed.  ***
***                           iwork(k) = 1 the maximum number of            ***
***                                        subdivisions (leniw - limlst)/2  ***
***                                        has been achieved on the k-th    ***
***                                        cycle.                           ***
***                                    = 2 occurrence of roundoff error is  ***
***                                        detected and prevents the        ***
***                                        tolerance imposed on the k th    ***
***                                        cycle, from being achieved on    ***
***                                        this cycle.                      ***
***                                    = 3 extremely bad integrand          ***
***                                        behaviour occurs at some points  ***
***                                        of the k th cycle.               ***
***                                    = 4 the integration procedure over   ***
***                                        the k th cycle does not converge ***
***                                        (to within the required          ***
***                                        accuracy) due to roundoff in the ***
***                                        extrapolation procedure invoked  ***
***                                        on this cycle. It is assumed     ***
***                                        that the result on this interval ***
***                                        is the best which can be         *** 
***                                        obtained.                        ***
***                                    = 5 the integral over the k th cycle ***
***                                        is probably divergent or slowly  ***
***                                        convergent.It must be noted that ***
***                                        divergence can occur with any    ***
***                                        other value of iwork(k).         ***
***                                                                         ***
***              if omega = 0 and integr = 1:                               ***
***                                                                         ***
***                           the integral is calculated by means of        ***
***                           cquad_dqagie, and ier = iwork(1) (with        ***
***                           meaning as described for iwork(k), k = 1).    ***
***                                                                         ***
***    lst     number of cycles actually needed for the integration.        ***
***            (if omega = 0, then lst is set to 1)                         ***
***-------------------------------------------------------------------------***
***  Workspace:                                                             ***
***                                                                         ***
***    limlst   limlst gives an upper bound on the number of cycles         ***
***             (limlst > 2)                                                ***
***                                                                         ***
***    leniw    dimensioning parameter for iwork.                           ***
***             (leniw - limlst) / 2 equals the maximum number of           ***
***             subintervals allowed in the partition of each cycle         ***
***             (leniw >= limlst + 2)                                       ***
***                                                                         ***
***    maxp1    gives an upper bound on the number of chebyshev moments     ***
***             which can be stored, i.e. for the intervals of lengths      ***
***             abs(b - a) * 2 * *(-l), l = 0, 1, ..., maxp1 - 2            ***
***             (maxp1 > 0)                                                 ***
***                                                                         ***
***    lenw     dimensioning value for work.                                ***
***             (lenw must be at least leniw * 2 + maxp1 * 25)              ***
***                                                                         ***
***    iwork    vector of dimension at least leniw.                         ***
***             iwork(k) for k = 1, 2, ..., lst contain the error flags on  ***
***             the cycles                                                  ***
***                                                                         *** 
***    work     vector of dimension at least lenw.                          ***
***             On return:                                                  ***
***                work(1), ..., work(lst) contain the integral             ***
***                  approximations over the cycles                         ***
***                work(limlst + 1), ..., work(limlst + lst) contain the    ***
***                  the error extimates over the cycles                    ***
***                Further elements of work have no specific meaning for    ***
***                  the user.                                              ***
***-------------------------------------------------------------------------***
***  References:      QUADPACK - Piessens, de Doncker, Uberhuber, Kahaner   ***
***                                                                         ***
***                   http://netlib.org                                     ***
******************************************************************************/

    int lenw = LENW, leniw = LENIW, limlst = 50, maxp1 = MAXP1;
    double res;

    lenw = LENW;
    leniw = LENIW;
    limlst = 50;
    maxp1 = MAXP1;

    dqawf_(f, &a, &omega, &integr, &epsabs, &res, abserr, neval, ier, &limlst, lst, &leniw, &maxp1, &lenw, iwork, work);

    return res;
}
